/**********************************************************************************************/
/* FBL_DateTimeEncode.h																	  	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma 1993-1999															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __FBL_DateTimeEncode_h 
	#define __FBL_DateTimeEncode_h

#if defined(__MWERKS__)
	#pragma options align=power
#endif

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#pragma mark -
#pragma mark === Date ===


/**********************************************************************************************/
#if FBL_BIG_ENDIAN	

union DateEncoded
{
	long  encoded;		// long value, the more date, the more value.
						// Dates can be <0, this means before 0 year.
	struct
	{
		signed   y	: 23;		// year		-2^22 to +2^22
		unsigned m	: 4;		// month, 	1 - 12
		unsigned d	: 5;		// day, 	1 - 31
	} decoded;
}; 

#else // For Windows (Intel processor) we need have back order of bit fields:

union DateEncoded
{
	long  encoded;		// long value, the more date, the more value.
						// Dates can be <0, this means before 0 year.
	struct
	{
		unsigned d	: 5;		// day, 	1 - 31
		unsigned m	: 4;		// month, 	1 - 12
		signed   y	: 23;		// year		-2^22 to +2^22
	} decoded;
}; 

#endif // FBL_BIG_ENDIAN


/**********************************************************************************************/
inline long EncodeDate( long inYear, ushort inMonth, ushort inDay )
{
	CHECK_FBL( inMonth <= 12 ); 
	CHECK_FBL( inDay <= 31 ); 

	DateEncoded date;
	
	date.decoded.y = inYear;
	date.decoded.m = inMonth;
	date.decoded.d = inDay;
	
	return date.encoded;
}

/**********************************************************************************************/
inline void DecodeDate( long inDate, long& outYear, ushort& outMonth, ushort& outDay )
{
	DateEncoded date;
	
	date.encoded = inDate;
	
	outYear 	= date.decoded.y;
	outMonth	= date.decoded.m;
	outDay		= date.decoded.d;	
}

#pragma mark -
#pragma mark === Time ===


/**********************************************************************************************/
#if FBL_BIG_ENDIAN	

union TimeEncoded
{
	ulong  encoded;	// ulong value, the more time, the more value.
	
	struct
	{
		unsigned h	: 20;	// hours		0 - 23
		unsigned m	: 6;	// minutes, 	0 - 59
		unsigned s	: 6;	// seconds, 	0 - 59
	} decoded;
};

#else // For Windows (Intel processor) we need have back order of bit fields:

union TimeEncoded
{
	ulong  encoded;	// ulong value, the more time, the more value.
	
	struct
	{
		unsigned s	: 6;	// seconds, 	0 - 59
		unsigned m	: 6;	// minutes, 	0 - 59
		unsigned h	: 20;	// hours		0 - 23
	} decoded;
};

#endif // FBL_BIG_ENDIAN


/**********************************************************************************************/
inline ulong EncodeTime( ushort inHour, ushort inMinutes, ushort inSeconds )
{
	CHECK_FBL( inHour <= 24 ); 
	CHECK_FBL( inMinutes <= 59 ); 
	CHECK_FBL( inSeconds <= 59 ); 

	TimeEncoded time;
	
	time.decoded.h = inHour;
	time.decoded.m = inMinutes;
	time.decoded.s = inSeconds;
	
	return time.encoded;
}

/**********************************************************************************************/
inline void DecodeTime( ulong inTime, ushort& outHour, ushort& outMinutes, ushort& outSeconds )
{
	TimeEncoded time;
		
	time.encoded = inTime;
	
	// Bug in CW 8.0 it seems not correctly inline this function with bit map.
	// it make hour as short always...On the other hand this is good idea to use 
	// bit operations always...why not?
	
	outHour 	= (ushort) ulong(inTime >> 12);
	outMinutes	= (ushort) time.decoded.m;
	outSeconds	= (ushort) time.decoded.s;	
}


#pragma mark -
#pragma mark === DateTime ===


/**********************************************************************************************/
#if FBL_BIG_ENDIAN

inline void FBL_EncodeDateTime( llong* outDateTime, long inDate, ulong inTime )
{
	long* p = (long*) outDateTime;
	
	p[0] = (long) inDate;
	p[1] = (long) inTime;
}

inline void FBL_DecodeDateTime( const llong* inDateTime, long* outDate, ulong* outTime )
{
	long* p = (long*) inDateTime;

	*outDate =  (long) p[0];
	*outTime = (ulong) p[1];
}

inline long FBL_DateTime_GetDate( const llong* inDateTime )
{
	long* p = (long*) inDateTime;
	return p[0];
}

inline ulong FBL_DateTime_GetTime( const llong* inDateTime )
{
	long* p = (long*) inDateTime;
	return (ulong) p[1];
}

inline void FBL_DateTime_SetDate( llong* ioDateTime, long inDate )
{
	long* p = (long*) ioDateTime;
	p[0] = inDate;
}

inline void FBL_DateTime_SetTime( llong* ioDateTime, ulong inTime )
{
	ulong* p = (ulong*) ioDateTime;
	p[1] = inTime;
}


/**********************************************************************************************/
#else // FBL_BIG_ENDIAN


inline void FBL_EncodeDateTime( llong* outDateTime, long inDate, ulong inTime )
{
	long* p = (long*) outDateTime;
	
	p[0] = (long) inTime;
	p[1] = (long) inDate;
}

inline void FBL_DecodeDateTime( const llong* inDateTime, long* outDate, ulong* outTime )
{
	long* p = (long*) inDateTime;

	*outDate =  (long) p[1];
	*outTime = (ulong) p[0];
}

inline long FBL_DateTime_GetDate( const llong* inDateTime )
{
	long* p = (long*) inDateTime;
	return p[1];
}

inline ulong FBL_DateTime_GetTime( const llong* inDateTime )
{
	long* p = (long*) inDateTime;
	return (ulong) p[0];
}

inline void FBL_DateTime_SetDate( llong* ioDateTime, long inDate )
{
	long* p = (long*) ioDateTime;
	p[1] = inDate;
}

inline void FBL_DateTime_SetTime( llong* ioDateTime, ulong inTime )
{
	ulong* p = (ulong*) ioDateTime;
	p[0] = inTime;
}

#endif // FBL_BIG_ENDIAN


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif	// __FBL_DateTimeEncode_h


