/**********************************************************************************************/
/* FBL_Thread.h																			  	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/
/*
	FBL_Thread class is a class with which work ALL rest kernel classes.
	
	FBL_Thread is wrapper to abstract FBL_ThreadNative class which specify interface of native 
	OS thread class. We use wrapping but not inheritance becuse Valentina SDK must be	  
	very flexiable -- work on different platforms and with different app frameworks.

	So if you have own framework which Valentina do not support on default you just need 
	subclass FBL_ThreadNative and provide own implementaition of thread class. You will
	use multiple inheritance. Look on FBL_PPThread as example.
	
	FBL_Thread class very tightly related with FBL_Task class.
	FBL_Task is abstract class which you should subclass to define your work for this thread.
	This is a comfortable way because in your subclass you can have complex data and complex result.
*/

#ifndef __FBL_Thread
	#define __FBL_Thread

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL_ThreadNative.h>
#else
#	include <FBL_ThreadNative.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
#ifdef FBL_WIN
	#define kFBL_ThreadOption_Default	0			// it is ignored under the Windows
#else
	#define kFBL_ThreadOption_Default	0x0004		// PP: threadOption_Alloc   = 0x0004, 	
#endif 


/**********************************************************************************************/
class FBL_Thread 
{
		friend class FBL_ThreadNative;

	public://///////////////////////////////////////////////////////////////////////////////////
 
static	void					InitStaticMembers( void );
static	void					DestroyStaticMembers( void );

		/* Construction/destruction: */

FBL_EXP							FBL_Thread(	
									FBL_Task* 	inTask,
									bool 		inOwner 	= true,
									ulong 		inStackSize = 0,
								 	ushort		inFlags 	= kFBL_ThreadOption_Default );

FBL_EXP virtual					~FBL_Thread( void );				
								  

		/* Execution methods: */

		void					Resume( void )	{ if(mThreadImp) mThreadImp->Resume(); }
		void					Suspend( void ) { if(mThreadImp) mThreadImp->Suspend(); }

								
		/* FBL_Task methods: */						 
		void					RunTask( void );		
								 
 
		/* Accessories: */
 
		ulong					GetID( void ) const { return mThreadImp->GetID(); }
				 
								
	protected:////////////////////////////////////////////////////////////////////////////////// 

 		FBL_Task*				GetTask( void ) const { return mTask; }						 								
		
								/** Destructor of native thread call this, so we must die also. */
		void					NativeThreadIsKilled( void ) { 
									mThreadImp = NULL;
									delete this;
								}
		

	protected:////////////////////////////////////////////////////////////////////////////////// 	
		
		FBL_ThreadNative*		mThreadImp;		// OS native thread implementation.
		
		FBL_Task*				mTask;			// Task which is executed in this thread.
		bool					mOwnTask;		// Is the task owned by this thread?
};


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif
